/*
RCon v1.10 Quake 2 Remote CONsole 
Copyright (c)1998 Michael Dwyer/N0ZAP (mdwyer@holly.colostate.edu)
18-May-1998

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <string.h>

int main(int argc, char *argv[])
   {
   int                socketfd;
   int                port = htons(27910);
   char               *portstr;
   unsigned long      ipnum;
   int                len,sent;
   int                i,j,k;
   struct sockaddr_in hostaddress;
   unsigned char      packet[256];
   unsigned char      header[10] = {0xff, 0xff, 0xff, 0xff,
                                   'r','c','o','n',0x20,0x00};


   /*
    * Command Line Sanity Checking
    */
   if (argc < 3)
      {
      fprintf(stderr,"RCon 1.1 by Michael Dwyer\n");
      fprintf(stderr,"Usage: %s <address>[:port] ", argv[0]);
      fprintf(stderr,"<password> commands ...\n\n");
      exit(1);
      }

   /*
    * Decode the address and port
    */
   portstr=strchr(argv[1],':');
   if (portstr)
      {
      *portstr++ = '\0';
      port=htons(atoi(portstr));
      fprintf(stderr,"Using port %i\n",ntohs(port));
      fprintf(stderr,"Using address %s\n", argv[1]);
      }
   else
      {
      fprintf(stderr,"Using port %i\n",ntohs(port));
      }
   ipnum=(inet_addr(argv[1]));
   if (ipnum==-1)
      {
      fprintf(stderr,"Invalid Address - Use xxx.xxx.xxx.xxx format.\n");
      fprintf(stderr,"I can't figure out domain names... \n");
      exit(1);
      }
   
   /*
    * Concat all the excess command line stuff into a single string
    */
   for (k=0;k<9;k++)
      {
      packet[k] = header[k];
      }
   for ( i=2 ; i<argc ; i++) 
      {
      for (j=0 ; j<strlen(argv[i]) ; j++)
         {
	 packet[k] = argv[i][j];
	 k++;
	 if (k>240)
	    {
	    fprintf(stderr,"Command too long\n");
	    exit(1);
	    }
	 }
      packet[k]=0x20;
      k++;
      }
   k--;
   packet[k]='\0';
   len = strlen(packet);
   len++;
   

   /*
    * Set the packet address info
    */
   bzero((char *) &hostaddress, sizeof(hostaddress));
   hostaddress.sin_family = AF_INET;
   hostaddress.sin_addr.s_addr = ipnum; 
   hostaddress.sin_port = port;

   /*
    * Open the Socket
    */
   if ((socketfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
      {
      perror("Couldn't open socket");
      exit(1);
      }

   /*
    * Send the packet
    */
   sent=sendto(socketfd, packet, len ,0, 
        (struct sockaddr *)&hostaddress, sizeof(hostaddress));

   /*
    * See if it worked
    */
   if (sent!=len)
      {
      perror("Sendto failed");
      fprintf(stderr,"Tried to send %i, sent %i\n",len,sent);
      close(socketfd);
      exit(1);
      }
   /*
    * Clean Up
    */
   close(socketfd);
   exit(0);
   }
